package com.pwt.netty.client;

import com.pwt.netty.handler.*;
import io.netty.bootstrap.Bootstrap;
import io.netty.buffer.ByteBuf;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;
import io.netty.handler.codec.LengthFieldBasedFrameDecoder;
import org.springframework.stereotype.Component;

import java.net.InetSocketAddress;
import java.util.Arrays;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import static com.pwt.framework.datasource.DynamicDataSourceContextHolder.log;

/**
 * @author linyun
 * @version V1.0
 * @Title: Vehicle2Client
 * @Description: netty客户端
 * @date 2024/03/24
 **/
@Component
public class DianBiaoClient {

    private Channel channel;
    private  Bootstrap bootstrap;

    public void startClient(String host, int port,boolean isTotal) {
        EventLoopGroup group = new NioEventLoopGroup();
        try {
            bootstrap = new Bootstrap();
            bootstrap.group(group)
                    .channel(NioSocketChannel.class)
                    .remoteAddress(new InetSocketAddress(host, port))
                    .option(ChannelOption.TCP_NODELAY, true);
            if(isTotal){
                //处理设备实时总数据
                bootstrap.handler(new ModbusTotalClientInitalizer());
            }else {
                //处理设备实时数据
                bootstrap.handler(new ModbusClientInitializer());
            }
            Channel channel = bootstrap.connect(host, port).sync().channel();
            // 发送 Modbus 请求数据

            ByteBuf buf = channel.alloc().buffer();
            buf.writeByte(0); // 事务标识符（高字节）
            buf.writeByte(1); // 事务标识符（低字节）
            buf.writeByte(0); // 协议标识符（高字节）
            buf.writeByte(0); // 协议标识符（低字节）
            buf.writeShort(12); // 长度
            buf.writeByte(1); // 单元标识符  serverID 1
            buf.writeByte(3); // 功能码（读取寄存器）
            if(isTotal){
                buf.writeShort(1536); // 起始寄存器地址(总值)
                buf.writeShort(12); // 寄存器数量
            }else {
                buf.writeShort(256); // 起始寄存器地址
                buf.writeShort(42); // 寄存器数量
            }
            channel.writeAndFlush(buf);

            // 在这里您可以做其他操作，比如等待服务器响应，关闭连接等
             channel.closeFuture().sync(); // 这里是阻塞的，会一直等待连接关闭
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            group.shutdownGracefully();
        }
    }
}
