package com.pwt.netty.client;

import com.ghgande.j2mod.modbus.io.ModbusTCPTransaction;
import com.ghgande.j2mod.modbus.msg.ReadMultipleRegistersRequest;
import com.ghgande.j2mod.modbus.msg.ReadMultipleRegistersResponse;
import com.ghgande.j2mod.modbus.net.TCPMasterConnection;
import com.pwt.common.utils.spring.SpringUtils;
import com.pwt.web.domain.DianBiaoData;
import com.pwt.web.service.impl.DianBiaoDataServiceImpl;
import lombok.extern.slf4j.Slf4j;

import java.net.InetAddress;

@Slf4j
public class ModbusTcpClient {

    // 设定MODBUS网络上从站地址
    private final static int SLAVE_ADDRESS = 1;
    //串行波特率
    private final static int BAUD_RATE = 9600;

    private static final String COM_PORT = "COM1";

    public void startClient(String host, int port, String deviceName ,boolean isTotal) {
        TCPMasterConnection connection = null;
        try {
            // 创建TCP连接
            InetAddress inetAddress = InetAddress.getByName(host);
            connection = new TCPMasterConnection(inetAddress);
            connection.setPort(port);
            connection.connect();
            // 创建ModbusTransaction实例
            ModbusTCPTransaction transaction = new ModbusTCPTransaction(connection);
            ReadMultipleRegistersRequest request = null;
            if(isTotal){
                request = new ReadMultipleRegistersRequest(1536, 12); // 从地址1536开始读取12个寄存器
            }else {
                request = new ReadMultipleRegistersRequest(256, 42); // 从地址256开始读取42个寄存器
            }
            request.setUnitID(1);
            transaction.setRequest(request);
            // 发送请求并获取响应
            transaction.execute();
            ReadMultipleRegistersResponse response = (ReadMultipleRegistersResponse) transaction.getResponse();
            // 处理响应数据
            if (response != null && response.getRegisters() != null) {
                String s = "";
                for(int i=0;i<response.getRegisters().length;i++){
                    s += ","+response.getRegisterValue(i);
                }
//                log.info(s.substring(1));
                DianBiaoData data = new DianBiaoData();
                data.setIp(host+":"+port);
                data.setDeviceName(deviceName);
                if(isTotal){
                    if (response.getRegisters().length < 12) {
                        //数据包长度不够，继续接收再处理
                        log.error("-----------数据包不完整，继续读取");
                        connection.close();
                        return;
                    }
                    data.setPositiveActiveEnergy(Long.valueOf((String.valueOf(response.getRegisterValue(0)) + String.valueOf(response.getRegisterValue(1))))/100);
//                    log.info("PositiveActiveEnergy的值,高字节:[{}]，低字节:[{}]，结果:[{}]",response.getRegisterValue(0),response.getRegisterValue(1),Long.valueOf((String.valueOf(response.getRegisterValue(0)) + String.valueOf(response.getRegisterValue(1))))/100);
                    data.setReverseActiveEnergy(Long.valueOf((String.valueOf(response.getRegisterValue(2)) + String.valueOf(response.getRegisterValue(3))))/100);
//                    log.info("ReverseActiveEnergy的值,高字节:[{}]，低字节:[{}]，结果:[{}]",response.getRegisterValue(2),response.getRegisterValue(3),Long.valueOf((String.valueOf(response.getRegisterValue(2)) + String.valueOf(response.getRegisterValue(9))))/100);
                    data.setForwardReactiveEnergy(Long.valueOf((String.valueOf(response.getRegisterValue(4)) + String.valueOf(response.getRegisterValue(5))))/100);
//                    log.info("ForwardReactiveEnergy的值,高字节:[{}]，低字节:[{}]，结果:[{}]",response.getRegisterValue(4),response.getRegisterValue(5),Long.valueOf((String.valueOf(response.getRegisterValue(4)) + String.valueOf(response.getRegisterValue(5))))/100);
                    data.setReverseReactiveEnergy(Long.valueOf((String.valueOf(response.getRegisterValue(6)) + String.valueOf(response.getRegisterValue(7))))/100);
//                    log.info("ReverseReactiveEnergy的值,高字节:[{}]，低字节:[{}]，结果:[{}]",response.getRegisterValue(6),response.getRegisterValue(7),Long.valueOf((String.valueOf(response.getRegisterValue(6)) + String.valueOf(response.getRegisterValue(7))))/100);
                    data.setTotalActivePower(Long.valueOf((String.valueOf(response.getRegisterValue(8)) + String.valueOf(response.getRegisterValue(9))))/100);
                    log.info("总有功电能的值,高字节:[{}]，低字节:[{}]，结果:[{}]",response.getRegisterValue(8),response.getRegisterValue(9),Long.valueOf((String.valueOf(response.getRegisterValue(8)) + String.valueOf(response.getRegisterValue(9))))/100);
                    data.setTotalReactivePower(Long.valueOf((String.valueOf(response.getRegisterValue(10)) + String.valueOf(response.getRegisterValue(11))))/100);
                    log.info("总无功电能的值,高字节:[{}]，低字节:[{}]，结果:[{}]",response.getRegisterValue(10),response.getRegisterValue(11),Long.valueOf((String.valueOf(response.getRegisterValue(10)) + String.valueOf(response.getRegisterValue(11))))/100);
                }
                else {
                    log.info("数据包长度："+ response.getRegisters().length);
                    if (response.getRegisters().length < 42) {
                        //数据包长度不够，继续接收再处理
                        log.error("-----------数据包不完整，继续读取");
                        connection.close();
                        return;
                    }
//                    for (int i = 0; i < response.getRegisters().length; i++) {
//                        log.info("Register " + (i + 1) + ": " + response.getRegisterValue(i));
//                    }//
                    data.setAphaseVoltage(Long.valueOf((String.valueOf(response.getRegisterValue(0)) + String.valueOf(response.getRegisterValue(1))))/100);
                    data.setBphaseVoltage(Long.valueOf((String.valueOf(response.getRegisterValue(2)) + String.valueOf(response.getRegisterValue(3))))/100);
                    data.setCphaseVoltage(Long.valueOf((String.valueOf(response.getRegisterValue(4)) + String.valueOf(response.getRegisterValue(5))))/100);
                    data.setAbLineVoltage(Long.valueOf((String.valueOf(response.getRegisterValue(6)) + String.valueOf(response.getRegisterValue(7))))/100);
                    data.setBcLineVoltage(Long.valueOf((String.valueOf(response.getRegisterValue(8)) + String.valueOf(response.getRegisterValue(9))))/100);
                    data.setCaLineVoltage(Long.valueOf((String.valueOf(response.getRegisterValue(10)) + String.valueOf(response.getRegisterValue(11))))/100);
                    data.setAphaseCurrent(Long.valueOf((String.valueOf(response.getRegisterValue(12)) + String.valueOf(response.getRegisterValue(13))))/1000);
                    log.info("A电流,高字节:[{}]，低字节:[{}]，结果:[{}]",response.getRegisterValue(12),response.getRegisterValue(13),Long.valueOf((String.valueOf(response.getRegisterValue(12)) + String.valueOf(response.getRegisterValue(13))))/1000);
                    data.setBphaseCurrent(Long.valueOf((String.valueOf(response.getRegisterValue(14)) + String.valueOf(response.getRegisterValue(15))))/1000);
                    log.info("B电流,高字节:[{}]，低字节:[{}]，结果:[{}]",response.getRegisterValue(14),response.getRegisterValue(15),Long.valueOf((String.valueOf(response.getRegisterValue(14)) + String.valueOf(response.getRegisterValue(15))))/1000);
                    data.setCphaseCurrent(Long.valueOf((String.valueOf(response.getRegisterValue(16)) + String.valueOf(response.getRegisterValue(17))))/1000);
                    log.info("C电流,高字节:[{}]，低字节:[{}]，结果:[{}]",response.getRegisterValue(16),response.getRegisterValue(17),Long.valueOf((String.valueOf(response.getRegisterValue(16)) + String.valueOf(response.getRegisterValue(17))))/1000);
                    data.setAphaseActivePower(Float.parseFloat((String.valueOf(response.getRegisterValue(18)) + String.valueOf(response.getRegisterValue(19))))/10000);
                    data.setBphaseActivePower(Float.parseFloat((String.valueOf(response.getRegisterValue(20)) + String.valueOf(response.getRegisterValue(21))))/10000);
                    data.setCphaseActivePower(Float.parseFloat((String.valueOf(response.getRegisterValue(22)) + String.valueOf(response.getRegisterValue(23))))/10000);
                    data.setTphaseActivePower(Float.parseFloat((String.valueOf(response.getRegisterValue(24)) + String.valueOf(response.getRegisterValue(25))))/10000);
                    data.setPhaseAreactivePower(Float.parseFloat((String.valueOf(response.getRegisterValue(26)) + String.valueOf(response.getRegisterValue(27))))/10000);
                    data.setPhaseBreactivePower(Float.parseFloat((String.valueOf(response.getRegisterValue(28)) + String.valueOf(response.getRegisterValue(29))))/10000);
                    data.setPhaseCreactivePower(Float.parseFloat((String.valueOf(response.getRegisterValue(30)) + String.valueOf(response.getRegisterValue(31))))/10000);
                    data.setPhaseTreactivePower(Float.parseFloat((String.valueOf(response.getRegisterValue(32)) + String.valueOf(response.getRegisterValue(33))))/10000);
                    data.setAlookEachPower(Float.parseFloat((String.valueOf(response.getRegisterValue(34)) + String.valueOf(response.getRegisterValue(35))))/10000);
                    data.setBlookEachPower(Float.parseFloat((String.valueOf(response.getRegisterValue(36)) + String.valueOf(response.getRegisterValue(37))))/10000);
                    data.setClookEachPower(Float.parseFloat((String.valueOf(response.getRegisterValue(38)) + String.valueOf(response.getRegisterValue(39))))/10000);
                    data.setTlookEachPower(Float.parseFloat((String.valueOf(response.getRegisterValue(40)) + String.valueOf(response.getRegisterValue(41))))/10000);
                }
                DianBiaoDataServiceImpl dianBiaoDataService = SpringUtils.getBean(DianBiaoDataServiceImpl.class);
                if(dianBiaoDataService.selectByIp(data.getIp())==null){
                    dianBiaoDataService.insertDianBiaoData(data);
                }else {
                    dianBiaoDataService.updateDianBiaoData(data);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // 关闭串口连接
            connection.close();
//            if (connection != null && connection.isConnected()) {
//                connection.close();
//            }
        }

        /*
        "COM3"——端口名
        BAUD_RATE——波特率
        8——数据位
        1——停止位
        0——校验位
        * */

//        SerialPortWrapper serialParameters = new
//                SerialPortWrapperImpl("COM1", BAUD_RATE, 8, 1, 0, 0, 0);
        /* 创建ModbusFactory工厂实例 */
//        ModbusFactory modbusFactory = new ModbusFactory();
        /* 创建ModbusMaster实例 */
//        IpParameters ipParameters = new IpParameters();
//        ipParameters.setHost(host);
//        ipParameters.setPort(port);
//        ModbusMaster master = modbusFactory.createTcpMaster(ipParameters,true);
        /* 初始化 */
//        try {
//            master.init();
//            //写入数据
//            short[] orders = {(short) 0x8002};
//            writeTest(master, SLAVE_ADDRESS, 0x4000, orders);
        //读取数据
//            readHoldingRegistersTest(master, SLAVE_ADDRESS, 0x0100, 42);
//        } catch (ModbusInitException e) {
//            e.printStackTrace();
//        } finally {
//            master.destroy();
//        }
    }


//    private void writeTest(ModbusMaster master,int slaveId, int startOffset, short[] sdata){
//        try {
//            WriteRegistersRequest request = new WriteRegistersRequest(slaveId,startOffset,sdata);
//            WriteRegistersResponse response = (WriteRegistersResponse) master.send(request);
//            if (response.isException()) {
//                System.out.println("Exception response: 写数据=" + response.getExceptionMessage());
//            } else {
//                System.out.println("CollectionMain写入1:" + response.getNumberOfRegisters());
//                System.out.println("CollectionMain写入2:" + response.getFunctionCode());
//            }
//        } catch (ModbusTransportException e) {
//            e.printStackTrace();
//        }
//    }
//
//    /**
//     * 读保持寄存器上的内容
//     * @param master 主站
//     * @param slaveId 从站地址
//     * @param start 起始地址的偏移量
//     * @param len 待读寄存器的个数
//     */
//    private void readHoldingRegistersTest(ModbusMaster master, int slaveId, int start, int len) {
//        try {
//            ReadHoldingRegistersRequest request = new ReadHoldingRegistersRequest(slaveId, start, len);
//            ReadHoldingRegistersResponse response = (ReadHoldingRegistersResponse)master.send(request);
//
//            if (response.isException()) {
//                System.out.println("Exception response: 读数据=" + response.getExceptionMessage());
//            } else {
//                System.out.println("CollectionMain输出:" + Arrays.toString(response.getShortData()));
//                short[] list = response.getShortData();
//                for (int i = 0; i < list.length; i++) {
//                    System.out.print(list[i] + " ");
//                }
//            }
//        } catch (ModbusTransportException e) {
//            e.printStackTrace();
//        }
//    }
}
