package com.pwt.netty.handler;

import com.alibaba.fastjson2.JSON;
import com.pwt.common.utils.spring.SpringUtils;
import com.pwt.netty.message.WebSocketMessage;
import com.pwt.netty.util.WebSocketUtil;
import com.pwt.web.domain.DianBiaoData;
import com.pwt.web.mapper.DianBiaoDataMapper;
import com.pwt.web.service.impl.DianBiaoDataServiceImpl;
import com.sun.media.jfxmedia.logging.Logger;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageDecoder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import static com.pwt.netty.conponents.WebSocketChannelRepository.channelMap;
import static com.pwt.netty.util.WebSocketUtil.pushMsgToOne;
@Component
@Slf4j
public class ModbusResponseDecoder extends ByteToMessageDecoder {

//    @Autowired
//    DianBiaoDataServiceImpl dianBiaoDataService;
    @Override
    protected void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out) {
        // 根据Modbus协议解析响应数据
        // 例如，假设响应数据格式为：03 04 00 01 00 02 crc1 crc2
        log.info("处理器3启动");
        int readableByte = in.readableBytes();
        if (in.readableBytes() < 42) {
            //数据包长度不够，继续接收再处理
            log.error("-----------数据包不完整，继续读取");
            return;
        }
        StringBuilder sb = new StringBuilder();
        List<Byte> resulte = new ArrayList<>();
        // 读取功能码
        int functionCode = in.readUnsignedByte();
        //回送数据域字节数(M)
        int feedbackAreaCode = in.readUnsignedByte();
        while (in.isReadable()) {
            resulte.add(in.readByte());
        }
        // 检查ArrayList的大小
        if (resulte.size() >= 2) {
            // 移除crc2
            resulte.remove(resulte.size() - 1);
            // 移除crc1
            resulte.remove(resulte.size() - 1);
        }
        Byte[] bytes = resulte.toArray(new Byte[0]);
        log.info("收到设备回传报文========="+String.valueOf(bytes));
        DianBiaoData data = new DianBiaoData();
        data.setIp(ctx.pipeline().channel().remoteAddress().toString());
        try {
            data.setAphaseVoltage(Long.getLong(bytes[0].toString() + bytes[1].toString()));
            data.setBphaseVoltage(Long.getLong(bytes[2].toString() + bytes[3].toString()));
            data.setCphaseVoltage(Long.getLong(bytes[4].toString() + bytes[5].toString()));
            data.setAbLineVoltage(Long.getLong(bytes[6].toString() + bytes[7].toString()));
            data.setBcLineVoltage(Long.getLong(bytes[8].toString() + bytes[9].toString()));
            data.setCaLineVoltage(Long.getLong(bytes[10].toString() + bytes[11].toString()));
            data.setAphaseCurrent(Long.getLong(bytes[12].toString() + bytes[13].toString()));
            data.setBphaseCurrent(Long.getLong(bytes[14].toString() + bytes[15].toString()));
            data.setCphaseCurrent(Long.getLong(bytes[16].toString() + bytes[17].toString()));
            data.setAphaseActivePower(Float.parseFloat((bytes[18].toString() + bytes[19].toString())));
            data.setBphaseActivePower(Float.parseFloat((bytes[20].toString() + bytes[21].toString())));
            data.setCphaseActivePower(Float.parseFloat((bytes[22].toString() + bytes[23].toString())));
            data.setTphaseActivePower(Float.parseFloat((bytes[24].toString() + bytes[25].toString())));
            data.setPhaseAreactivePower(Float.parseFloat((bytes[26].toString() + bytes[27].toString())));
            data.setPhaseBreactivePower(Float.parseFloat((bytes[28].toString() + bytes[29].toString())));
            data.setPhaseCreactivePower(Float.parseFloat((bytes[30].toString() + bytes[31].toString())));
            data.setPhaseTreactivePower(Float.parseFloat((bytes[32].toString() + bytes[33].toString())));
            data.setAlookEachPower(Float.parseFloat((bytes[34].toString() + bytes[35].toString())));
            data.setBlookEachPower(Float.parseFloat((bytes[36].toString() + bytes[37].toString())));
            data.setClookEachPower(Float.parseFloat((bytes[38].toString() + bytes[39].toString())));
            data.setTlookEachPower(Float.parseFloat((bytes[40].toString() + bytes[41].toString())));
        } catch (Exception e) {
            log.error("当前回文报文异常: " + e.getMessage());
        }
        log.info(data.toString());
//        DianBiaoDataServiceImpl dianBiaoDataService = SpringUtils.getBean(DianBiaoDataServiceImpl.class);
//        if(dianBiaoDataService.selectByIp(data.getIp())==null){
//            dianBiaoDataService.insertDianBiaoData(data);
//        }else {
//            dianBiaoDataService.updateDianBiaoData(data);
//        }
        out.add(data);
        // TODO 发送给前端
//        WebSocketUtil.pushMsgToAll(WebSocketMessage.build("aaa", data));
        log.info("websocket发送：" + JSON.toJSON(data));

    }

    // 将字节数组转换为十六进制字符串
    private String byteArrayToHexString(byte[] bytes) {
        StringBuilder sb = new StringBuilder();
        for (byte b : bytes) {
            sb.append(String.format("%02X ", b));
        }
        return sb.toString();
    }
}
