package com.pwt.netty.handler;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.channel.SimpleChannelInboundHandler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

@Component
@Slf4j
class ModbusResponseHandler extends SimpleChannelInboundHandler<ByteBuf> {

    @Override
    protected void channelRead0(ChannelHandlerContext channelHandlerContext, ByteBuf msg) throws Exception {
        // 处理接收到的服务器返回数据
        byte[] response = new byte[msg.readableBytes()];
        msg.readBytes(response);
        log.info("Received response: " + bytesToHex(response));
        // 在这里处理接收到的 Modbus TCP 响应数据，根据实际情况来处理
    }

//    @Override
//    public void channelActive(ChannelHandlerContext ctx) {
//        // 发送Modbus TCP请求
//        ByteBuf requestBuffer = ctx.alloc().buffer();
//        // 填充请求缓冲区
////        byte[] msgBytes = {0x01, 0x03, 0x0, (byte) 0xFF, 0x00, 0x2A, (byte) crc16(new byte[]{0x01, 0x03, 0x01, 0x00, 0x00, 0x2A}, 6), (byte) crc16(new byte[]{0x01, 0x03, 0x01, 0x00, 0x00, 0x2A, (byte) crc16(new byte[]{0x01, 0x03, 0x01, 0x00, 0x00, 0x2A}, 6)}, 7)};
////        byte[] msgBytesTotal = {0x01, 0x03, 0x05, (byte) 0xFF, 0x00, 0x0C, (byte) crc16(new byte[]{0x01, 0x03, 0x05, (byte) 0xFF, 0x00, 0x0C}, 6), (byte) crc16(new byte[]{0x01, 0x03, 0x05, (byte) 0xFF, 0x00, 0x0C, (byte) crc16(new byte[]{0x01, 0x03, 0x05, (byte) 0xFF, 0x00, 0x0C,}, 6)}, 7)};
//        // 例如，发送一个简单的Modbus TCP读取线圈状态请求（功能码1）
//        requestBuffer.writeByte(0); // 事务标识符（高字节）
//        requestBuffer.writeByte(1); // 事务标识符（低字节）
//        requestBuffer.writeByte(0); // 协议标识符（高字节）
//        requestBuffer.writeByte(0); // 协议标识符（低字节）
//        requestBuffer.writeShort(12); // 长度
//        requestBuffer.writeByte(1); // 单元标识符
//        requestBuffer.writeByte(3); // 功能码寄存器
//        requestBuffer.writeShort(256); // 寄存器起始地址
//        requestBuffer.writeShort(42); // 寄存器数量
//
//        // 发送请求到服务器
//        ctx.writeAndFlush(requestBuffer);
//    }

    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) {
        // 处理Modbus TCP响应
        log.info("数据处理器启动");
        ByteBuf responseBuffer = (ByteBuf) msg;
        // 处理响应数据

        // 例如，打印响应数据
        byte[] responseData = new byte[responseBuffer.readableBytes()];
        responseBuffer.readBytes(responseData);
        System.out.println("Received response: " + bytesToHex(responseData));

        // 释放缓冲区
        responseBuffer.release();
    }

    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        cause.printStackTrace();
        ctx.close();
    }

    // 辅助方法，将字节数组转换为十六进制字符串
    private static String bytesToHex(byte[] bytes) {
        StringBuilder hexString = new StringBuilder();
        for (byte b : bytes) {
            hexString.append(String.format("%02X ", b));
        }
        return hexString.toString();
    }
}
