package com.pwt.netty.handler;

import com.alibaba.fastjson2.JSON;
import com.pwt.common.utils.spring.SpringUtils;
import com.pwt.web.domain.DianBiaoData;
import com.pwt.web.service.impl.DianBiaoDataServiceImpl;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageDecoder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import static com.pwt.netty.conponents.WebSocketChannelRepository.channelMap;
import static com.pwt.netty.util.WebSocketUtil.pushMsgToAll;
import static com.pwt.netty.util.WebSocketUtil.pushMsgToOne;

@Component
@Slf4j
public class ModbusResponseTotalDecoder extends ByteToMessageDecoder{

//    @Autowired
//    DianBiaoDataServiceImpl dianBiaoDataService;

    @Override
    protected void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> list) throws Exception {
        // 根据Modbus协议解析响应数据
        // 例如，假设响应数据格式为：03 04 00 01 00 02 crc1 crc2
        log.info("总和值处理器3启动");
        int readableByte = in.readableBytes();
        if(in.readableBytes() < 16) {
            //数据包长度不够，继续接收再处理
            log.error("-----------数据包不完整，继续读取");
            return;
        }
        StringBuilder sb = new StringBuilder();
        List<Byte> resulte = new ArrayList<>();
        // 读取功能码
        int functionCode = in.readUnsignedByte();
        //回送数据域字节数(M)
        int feedbackAreaCode = in.readUnsignedByte();
        while (in.isReadable()) {
            resulte.add(in.readByte());
        }
        // 检查ArrayList的大小
        if (resulte.size() >= 2) {
            // 移除crc2
            resulte.remove(resulte.size() - 1);
            // 移除crc1
            resulte.remove(resulte.size() - 1);
        }
        Byte[] bytes = resulte.toArray(new Byte[0]);
        log.info("收到设备回传报文========="+String.valueOf(bytes));
        DianBiaoData data = new DianBiaoData();
        data.setIp(ctx.pipeline().channel().remoteAddress().toString());
        try {
            data.setPositiveActiveEnergy(Long.valueOf((bytes[0].toString() + bytes[1].toString())));
            data.setReverseActiveEnergy(Long.valueOf((bytes[2].toString() + bytes[3].toString())));
            data.setForwardReactiveEnergy(Long.valueOf((bytes[4].toString() + bytes[5].toString())));
            data.setReverseReactiveEnergy(Long.valueOf((bytes[6].toString() + bytes[7].toString())));
            data.setTotalActivePower(Long.valueOf((bytes[8].toString() + bytes[9].toString())));
            data.setTotalReactivePower(Long.valueOf((bytes[10].toString() + bytes[11].toString())));
        }catch (Exception e){
            log.error("当前回文报文异常: " + e.getMessage());
        }
        log.info(data.toString());
//        DianBiaoDataServiceImpl dianBiaoDataService = SpringUtils.getBean(DianBiaoDataServiceImpl.class);
//        if(dianBiaoDataService.selectByIp(data.getIp())==null){
//            dianBiaoDataService.insertDianBiaoData(data);
//        }else {
//            dianBiaoDataService.updateDianBiaoData(data);
//        }
//        pushMsgToAll(properties);
        log.info("websocket发送："+ JSON.toJSON(data));
    }
}
